﻿namespace U2UC.WinRT.LexDbSample.DataAccessLayer
{
    using Helpers;
    using Lex.Db;
    using Models;
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Threading.Tasks;
    using Windows.ApplicationModel;
    using Windows.Storage;

    internal static class Dal
    {
        private static DbInstance db;

        static Dal()
        {
            // Create database
            db = new DbInstance("storage");

            // Define table mapping
            // * 1st parameter is primary key
            // * 2nd parameter is autoGen e.g. auto-increment
            db.Map<Person>().Automap(p => p.Id, true); 
            
            // Initialize database
            db.Initialize();
        }

        public static async Task CreateDatabase()
        {
            // The DB and the Person mapping were created in the static constructor.

            // Clear tabe
            db.Table<Person>().Purge();

            // (Re-)Populate table
            List<Person> persons = new List<Person>();
            Person person = new Person();
            person.Id = 1;
            person.Name = "11th Doctor";
            person.Status = 0;
            person.Description = "The Eleventh Doctor was a capricious and adventurous incarnation of the renegade Time Lord known as the Doctor.";
            StorageFile file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\11th_Doctor.png"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            person = new Person();
            person.Id = 2;
            person.Name = "Weeping Angel";
            person.Status = 2;
            person.Description = "The Weeping Angels were a species of quantum-locked humanoids, so called because their unique nature necessitated that they often covered their faces with their hands to prevent trapping each other in petrified form for eternity by looking at one another.";
            file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Weeping_Angel.jpg"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            person = new Person();
            person.Id = 3;
            person.Name = "Silent";
            person.Status = 2;
            person.Description = "The Silents were a 'memory-proof' alien species who led a religious order known as the Silence. They seemed to be willing to do anything to kill the Doctor before he could answer the Question, including murder and kidnapping.";
            file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Silence.jpg"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            person = new Person();
            person.Id = 4;
            person.Name = "River Song";
            person.Status = 1;
            person.Description = "'River Song', the primary alias of the third incarnation of Melody Pond, was a 'child of the TARDIS' and a known wife of the Eleventh Doctor. She was human with Time Lord characteristics, conceived by her parents, Amy Pond and Rory Williams, aboard the TARDIS as it travelled through the time vortex, and then conditioned by the Silence.";
            file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\River_Song.jpg"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            person = new Person();
            person.Id = 5;
            person.Name = "Rory Williams";
            person.Status = 0;
            person.Description = "Rory Arthur Williams — sometimes called Rory Pond — was Amy Pond's husband. He became a companion of the Eleventh Doctor on the night before their wedding, but he died and was erased from history after being absorbed by the Time Field.";
            file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Rory.jpg"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            person = new Person();
            person.Id = 6;
            person.Name = "Dalek";
            person.Status = 2;
            person.Description = "Daleks were the armoured, mutated descendants of the Kaleds of the planet Skaro. They fought the Time Lords in the Last Great Time War, ending in the near-total destruction of both races.";
            file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Daleks.jpg"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            person = new Person();
            person.Id = 7;
            person.Name = "Cyberman";
            person.Status = 0;
            person.Description = "Cybermen were a 'race' of cybernetically augmented humanoids. They varied greatly in design, with different factions throughout time and space.";
            file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Cybermen.jpg"));
            person.Picture = await file.AsByteArray();
            persons.Add(person);

            db.Table<Person>().Save(persons);
        }

        public static List<Person> GetAllPersons()
        {
            return db.Table<Person>().LoadAll().ToList();
        }

        public static Person GetPersonById(int Id)
        {
            return db.Table<Person>().LoadByKey(Id);
        }

        public static void SavePerson(Person person)
        {
            db.Table<Person>().Save(person);
        }

        public static void DeletePerson(Person person)
        {
            db.Table<Person>().Delete(person);
        }
    }
}